@@ -1,14 +1,14 @@
-/* @(#)parse.c	1.108 12/12/20 Copyright 1985-2012 J. Schilling */
+/* @(#)parse.c	1.114 15/03/12 Copyright 1985-2015 J. Schilling */
 #include <schily/mconfig.h>
 #ifndef lint
 static	UConst char sccsid[] =
-	"@(#)parse.c	1.108 12/12/20 Copyright 1985-2012 J. Schilling";
+	"@(#)parse.c	1.114 15/03/12 Copyright 1985-2015 J. Schilling";
 #endif
 /*
  *	Make program
  *	Parsing routines
  *
- *	Copyright (c) 1985-2012 by J. Schilling
+ *	Copyright (c) 1985-2015 by J. Schilling
  */
 /*
  * The contents of this file are subject to the terms of the
@@ -17,6 +17,8 @@ static	UConst char sccsid[] =
  * with the License.
  *
  * See the file CDDL.Schily.txt in this distribution for details.
+ * A copy of the CDDL is also available via the Internet at
+ * http://www.opensource.org/licenses/cddl1.txt
  *
  * When distributing Covered Code, include this CDDL HEADER in each
  * file and include the License file CDDL.Schily.txt from this distribution.
@@ -65,6 +67,7 @@ LOCAL	int	exp_ovec	__PR((obj_t ** ovec, int objcnt));
 LOCAL	int	read_ovec	__PR((obj_t ** ovec, int *typep));
 EXPORT	list_t	*cvtvpath	__PR((list_t *l));
 EXPORT	BOOL	is_inlist	__PR((char *objname, char *name));
+LOCAL	BOOL	obj_in_list	__PR((obj_t *obj, list_t *l));
 EXPORT	BOOL	nowarn		__PR((char *name));
 LOCAL	void	warn		__PR((char *, ...)) __printflike__(1, 2);
 LOCAL	void	exerror		__PR((char *, ...)) __printflike__(1, 2);
@@ -110,6 +113,7 @@ parsefile()
 	if (Dmake > 0)
 		error(">>>>>>>>>>>>>>>> Reading makefile '%s'\n", mfname);
 
+	ovec[0] = 0;
 	lineno = 1;
 	col = 0;
 	getch();
@@ -226,7 +230,7 @@ printf("\n");
 		 */
 /*		if (basetype(type) == COLON) {*/
 		if (basetype(type) == COLON ||
-		    (basetype(type) == EQUAL && deplist &&
+		    (basetype(type) == EQUAL && deplist && objcnt > 0 &&
 					ovec[0]->o_name[0] == '.')) {
 			cmdlist = getcmd();
 		} else {
@@ -258,6 +262,13 @@ define_obj(obj, n, objcnt, type, dep, cmd)
 	register list_t	*dep;
 		cmd_t	*cmd;
 {
+	/*
+	 * First check for possible direct recursions to avoid to blow
+	 * up memory immediately.
+	 */
+	if (obj_in_list(obj, dep))
+		exerror("Recursion in dependencies for '%s'", obj->o_name);
+
 	/*
 	 * If we have a list of targets with the same dependency list,
 	 * we copy the list structure to be able to separately
@@ -560,7 +571,7 @@ check_ssuffrule(obj, dep)
 	extern	 int	xssrules;
 
 	/*
-	 * Check if the first charater of the target is a '.' or
+	 * Check if the first character of the target is a '.' or
 	 * if the target name equals "", but make sure this is
 	 * not a vanilla target that just starts with "./".
 	 */
@@ -724,6 +735,8 @@ getobjname()
 		n++;
 		getch();
 	}
+	if (p >= gbufend)
+		p = growgbuf(p);
 	*p = '\0';				/* Terminate with null char */
 	return (n);
 }
@@ -795,6 +808,8 @@ getname(type)
 		n++;
 		getch();
 	}
+	if (p >= gbufend)
+		p = growgbuf(p);
 	*p = '\0';				/* Terminate with null char */
 	return (n);
 }
@@ -833,6 +848,8 @@ getln()
 		n++;
 		getch();
 	}
+	if (p >= gbufend)
+		p = growgbuf(p);
 	*p = '\0';				/* Terminate with null char */
 	return (n);
 }
@@ -1108,6 +1125,8 @@ getcmd()
 				p = growgbuf(p);
 			*p++ = lastc;
 		}
+		if (p >= gbufend)
+			p = growgbuf(p);
 		*p = '\0';
 		*tail = item = (cmd_t *) fastalloc(sizeof (cmd_t));
 		item->c_line = strsave(gbuf);
@@ -1232,8 +1251,8 @@ read_ovec(ovec, typep)
 			*typep = ASSIGN;
 			if (!nowarn(":=")) {
 				warn(
-				"Nonportable ':=' assignement found for macro '%s'.\n",
-				o->o_name);
+				"Nonportable ':=' assignment found for macro '%s'",
+				o ? o->o_name : "<empty>");
 			}
 			break;
 		}
@@ -1266,10 +1285,13 @@ cvtvpath(l)
 	list_t	*list = (list_t *)0;
 	list_t	**tail = &list;
 	char	vpath[NAMEMAX];
+	char	*vpe = &vpath[NAMEMAX-1];
 	char	*p1, *p2;
 
 	if (l != NULL) {
 		for (p1 = l->l_obj->o_name, p2 = vpath; *p1; p2++) {
+			if (p2 >= vpe)
+				break;
 			if ((*p2 = *p1++) == ':')
 				*p2 = ' ';
 		}
@@ -1307,6 +1329,19 @@ is_inlist(objname, name)
 	return (FALSE);
 }
 
+LOCAL BOOL
+obj_in_list(obj, l)
+	register obj_t	*obj;
+	register list_t	*l;
+{
+	while (l) {
+		if (obj == l->l_obj)
+			return (TRUE);
+		l = l->l_next;
+	}
+	return (FALSE);
+}
+
 EXPORT BOOL
 nowarn(name)
 	char	*name;
