@@ -1,14 +1,14 @@
-/* @(#)update.c	1.126 14/03/25 Copyright 1985, 88, 91, 1995-2014 J. Schilling */
+/* @(#)update.c	1.136 15/03/14 Copyright 1985, 88, 91, 1995-2015 J. Schilling */
 #include <schily/mconfig.h>
 #ifndef lint
 static	UConst char sccsid[] =
-	"@(#)update.c	1.126 14/03/25 Copyright 1985, 88, 91, 1995-2014 J. Schilling";
+	"@(#)update.c	1.136 15/03/14 Copyright 1985, 88, 91, 1995-2015 J. Schilling";
 #endif
 /*
  *	Make program
  *	Macro handling / Dependency Update
  *
- *	Copyright (c) 1985, 88, 91, 1995-2014 by J. Schilling
+ *	Copyright (c) 1985, 88, 91, 1995-2015 by J. Schilling
  */
 /*
  * The contents of this file are subject to the terms of the
@@ -75,6 +75,7 @@ EXPORT	char	*substitute	__PR((char *cmd, obj_t * obj, obj_t * source, char *suff
 LOCAL	char	*subst		__PR((char *cmd, obj_t * obj, obj_t * source, char *suffix, list_t * depends));
 LOCAL	char	*dynmac		__PR((char *cmd, obj_t * obj, obj_t * source, char *suffix, list_t * depends, BOOL  domod));
 LOCAL	void	warn_implicit	__PR((obj_t *obj, char *mac, char *exp));
+LOCAL	void	warn_displaced	__PR((obj_t *obj, char *mac, char *exp));
 LOCAL	void	extr_filenames	__PR((char *names));
 LOCAL	void	extr_dirnames	__PR((char *names));
 LOCAL	char	*exp_var	__PR((char end, char *cmd, obj_t * obj, obj_t * source, char *suffix, list_t *depends));
@@ -257,7 +258,7 @@ isprecious(obj)
 
 /*
  * Return TRUE if 'obj' is in the list of targets that should not be checked
- * aginst existing files. A .PHONY target is asumed to be never up to date,
+ * aginst existing files. A .PHONY target is assumed to be never up to date,
  * it is not removed in case a signal is received.
  */
 EXPORT BOOL
@@ -531,6 +532,9 @@ list_nth(list, n)
 	register list_t *list;
 		int	n;
 {
+	if (n < 0)
+		return ((list_t *)NULL);
+
 	for (; list; list = list->l_next)
 		if (--n < 0)
 			return (list);
@@ -783,8 +787,10 @@ subst(cmd, obj, source, suffix, depends)
 			/*
 			 * No need to update 'sb' as we exit here...
 			 */
-			if (sb != gbuf)
+			if (sb != gbuf) {
 				sp = gbuf + (sp - sb);
+				sb = gbuf;
+			}
 			comerrno(EX_BAD,
 				"Fatal error '$' at end of string '%s$'\n",
 									sp);
@@ -803,8 +809,10 @@ subst(cmd, obj, source, suffix, depends)
 	sub_s_put(cmd);
 	*sub_ptr = '\0';
 	depth--;
-	if (sb != gbuf)
+	if (sb != gbuf) {
 		sp = gbuf + (sp - sb);
+		sb = gbuf;
+	}
 	return (sp);
 }
 
@@ -830,6 +838,7 @@ dynmac(cmd, obj, source, suffix, depends, domod)
 	char	*sp1;
 	char	*sb1;
 	register char	*p = cmd;
+	char	*ocmd;
 
 	switch (*cmd++) {
 
@@ -877,6 +886,8 @@ dynmac(cmd, obj, source, suffix, depends, domod)
 		break;
 	case '0': case '1': case '2': case '3': case '4':
 	case '5': case '6': case '7': case '8': case '9':
+		if (depends->l_obj == NULL && obj->o_type == 0 && !nowarn("$#"))
+			warn_displaced(obj, "$#", "");
 		cmd = astoi(p, &num);		/* $1 -> first dependency */
 		sub_arg(num, depends, (obj_t *)0); /* $0 -> implicit source  */
 		break;
@@ -885,9 +896,22 @@ dynmac(cmd, obj, source, suffix, depends, domod)
 		 * $r0 -> all dependencies + implicit source
 		 * $r1 -> all dependencies
 		 */
+		if (depends->l_obj == NULL && obj->o_type == 0 &&
+		    !nowarn("$r#"))
+			warn_displaced(obj, "$r#", "");
 		sp1 = sub_ptr;
 		sb1 = gbuf;
+		ocmd = cmd;
 		cmd = astoi(cmd, &num);
+		if (num < 0 || cmd == ocmd) {
+			/*
+			 * sub_arg() deals with negative "num" values, but
+			 * while (num++) could nearly take forever.
+			 */
+			errmsgno(EX_BAD,
+			"WARNING: illegal substitution: '$r%s'\n", ocmd);
+			break;
+		}
 		while (sub_arg(num++, depends, (obj_t *)0)) {
 			if (sb1 != gbuf)
 				sp1 = gbuf + (sp1 - sb1);
@@ -898,6 +922,7 @@ dynmac(cmd, obj, source, suffix, depends, domod)
 		}
 		if (sb != gbuf)
 			sp = gbuf + (sp - sb);
+		sb = gbuf;
 		if (sp != sub_ptr && sub_ptr[-1] == ' ')
 			sub_ptr--;		/* Kill last space */
 		break;
@@ -915,6 +940,7 @@ dynmac(cmd, obj, source, suffix, depends, domod)
 		}
 		if (sb != gbuf)
 			sp = gbuf + (sp - sb);
+		sb = gbuf;
 		if (sp != sub_ptr && sub_ptr[-1] == ' ')
 			sub_ptr--;		/* Kill last space */
 		break;
@@ -932,6 +958,7 @@ dynmac(cmd, obj, source, suffix, depends, domod)
 		}
 		if (sb != gbuf)
 			sp = gbuf + (sp - sb);
+		sb = gbuf;
 		if (sp != sub_ptr && sub_ptr[-1] == ' ')
 			sub_ptr--;		/* Kill last space */
 		break;
@@ -943,11 +970,11 @@ dynmac(cmd, obj, source, suffix, depends, domod)
 		return (cmd);
 	if (sb != gbuf)
 		sp = gbuf + (sp - sb);
+	sb = gbuf;
 	if (*cmd == 'F') {
 		extr_filenames(sp);		/* 'sp' must be in gbuf */
 		return (++cmd);
-	}
-	if (*cmd == 'D') {
+	} else if (*cmd == 'D') {
 		extr_dirnames(sp);		/* 'sp' must be in gbuf */
 		return (++cmd);
 	}
@@ -971,6 +998,23 @@ warn_implicit(obj, mac, exp)
 				curwdir(), MakeFileNames[obj->o_fileindex]);
 }
 
+LOCAL void
+warn_displaced(obj, mac, exp)
+	obj_t	*obj;
+	char	*mac;
+	char	*exp;
+{
+	errmsgno(EX_BAD,
+	"WARNING: requesting dynmac '%s' in target definition\n",
+			mac);
+	errmsgno(EX_BAD,
+		"WARNING: expanding implicit dynmac  '%s' to '%s'\n",
+			mac, exp);
+	errmsgno(EX_BAD,
+		"WARNING: Current working directory:  '%s', Makefile '%s'\n",
+				curwdir(), mfname);
+}
+
 /*
  * Extract the filename parts from a string that contains a list of names.
  *
@@ -988,22 +1032,49 @@ extr_filenames(names)
 	char	*s;
 	char	*sp;
 	char	*sb;
+	char	*sbe;
 
 	sp = ++sub_ptr;
 	sb = gbuf;
+	sbe = gbufend;
 	/*
 	 * Make sure that gbuf has enough room for the copy.
+	 * Getting the size with sub_ptr - names only works if "names"
+	 * is on gbuf.
 	 */
-	grant_gbuf(sub_ptr - names);
+	if (wason_gbuf(names))
+		grant_gbuf(sub_ptr - names);
 	if (sb != gbuf) {
 		sp = gbuf + (sp - sb);
-		names = gbuf + (names - sb);
+		if (wason_gbuf(names))
+			names = gbuf + (names - sb);
+		sb = gbuf;
+		sbe = gbufend;
 	}
 	for (np = p = names, s = sp; np && *np; p = np) {
 		np = strchr(np, ' ');
 		if (np)
 			*np++ = '\0';
 		p = filename(p);
+
+		/*
+		 * Make sure the next string fits on gbuf.
+		 */
+		sub_ptr = s;		/* needed by grant_gbuf() */
+		grant_gbuf(strlen(p)+2); /* add null-byte and space */
+		if (sb != gbuf) {
+			sp = gbuf + (sp - sb);
+			s = gbuf + (s - sb);
+			if (wason_gbuf(names)) {
+				names = gbuf + (names - sb);
+				p = gbuf + (p - sb);
+				if (np)
+					np = gbuf + (np - sb);
+			}
+			sb = gbuf;
+			sbe = gbufend;
+		}
+
 		while (*p)
 			*s++ = *p++;
 		if (np)
@@ -1011,12 +1082,13 @@ extr_filenames(names)
 	}
 	*s = '\0';
 	/*
-	 * Now copy down from uppe part of gbuf.
+	 * Now copy down from upper part of gbuf.
 	 */
 	for (s = names, p = sp; *p; )
 		*s++ = *p++;
 	*s = '\0';
-	sub_ptr = s;
+	if (wason_gbuf(names))
+		sub_ptr = s;
 }
 
 /*
@@ -1036,6 +1108,7 @@ extr_dirnames(names)
 	char	*s;
 	char	*sp;
 	char	*sb;
+	char	*sbe;
 	char	_base[TYPICAL_NAMEMAX];
 	char	*base = _base;
 	char	*bp = NULL;
@@ -1044,13 +1117,20 @@ extr_dirnames(names)
 
 	sp = ++sub_ptr;
 	sb = gbuf;
+	sbe = gbufend;
 	/*
 	 * Make sure that gbuf has enough room for the copy.
+	 * Getting the size with sub_ptr - names only works if "names"
+	 * is on gbuf.
 	 */
-	grant_gbuf(sub_ptr - names);
+	if (wason_gbuf(names))
+		grant_gbuf(sub_ptr - names);
 	if (sb != gbuf) {
 		sp = gbuf + (sp - sb);
-		names = gbuf + (names - sb);
+		if (wason_gbuf(names))
+			names = gbuf + (names - sb);
+		sb = gbuf;
+		sbe = gbufend;
 	}
 	for (np = p = names, s = sp; np && *np; p = np) {
 		np = strchr(np, ' ');
@@ -1060,6 +1140,24 @@ extr_dirnames(names)
 		} else {
 			len = strlen(p) + 1;
 		}
+
+		/*
+		 * Make sure the next string fits on gbuf.
+		 */
+		sub_ptr = s;		/* needed by grant_gbuf() */
+		grant_gbuf(len+1);	/* add null-byte and space */
+		if (sb != gbuf) {
+			sp = gbuf + (sp - sb);
+			s = gbuf + (s - sb);
+			if (wason_gbuf(names)) {
+				names = gbuf + (names - sb);
+				p = gbuf + (p - sb);
+				if (np)
+					np = gbuf + (np - sb);
+			}
+			sb = gbuf;
+			sbe = gbufend;
+		}
 		if (len > blen) {
 			blen = len + 32;	/* Add some reserve */
 			base = bp = ___realloc(bp, blen, "dir base");
@@ -1080,7 +1178,8 @@ extr_dirnames(names)
 	for (s = names, p = sp; *p; )
 		*s++ = *p++;
 	*s = '\0';
-	sub_ptr = s;
+	if (wason_gbuf(names))
+		sub_ptr = s;
 }
 
 #define	white(c)	((c) == ' ' || (c) == '\t')
@@ -1088,7 +1187,7 @@ extr_dirnames(names)
 /*
  * Expand a macro.
  * As the replacement may be a suffix rule or a pattern rule too,
- * we first must get the basic name the macro referres to.
+ * we first must get the basic name the macro refers to.
  */
 #ifdef	PROTOTYPES
 LOCAL char *
@@ -1153,8 +1252,10 @@ exp_var(end, cmd, obj, source, suffix, depends)
 			name = ___realloc(np, nlen, "macro name");
 			if (np == NULL) {
 				/*
-				 * Copy old content
+				 * Copy old content.
+				 * First null-terminate the current buffer.
 				 */
+				*rname = '\0';
 				strlcpy(name, _name, sizeof (_name));
 				rname = name + (rname - _name);
 			} else {
@@ -1199,8 +1300,11 @@ exp_var(end, cmd, obj, source, suffix, depends)
 				pat = ___realloc(pp, plen, "macro pattern");
 				if (pp == NULL) {
 					/*
-					 * Copy old content
+					 * Copy old content.
+					 * First null-terminate the
+					 * current buffer.
 					 */
+					*rname = '\0';
 					strlcpy(pat, _pat, sizeof (_pat));
 					rname = pat + (rname - _pat);
 				} else {
@@ -1224,7 +1328,7 @@ exp_var(end, cmd, obj, source, suffix, depends)
 		found_make = TRUE;
 
 	/*
-	 * If the name of the macro contains a '$', resursively expand it.
+	 * If the name of the macro contains a '$', recursively expand it.
 	 * We need to check if we should rather expand anything between the
 	 * brackets (e.g {...}) however, this may fail to expand long lists.
 	 * See also comment in exp_name() regarding USE_SUBPAT
@@ -1239,6 +1343,7 @@ exp_var(end, cmd, obj, source, suffix, depends)
 		s2 = subst(name, obj, source, suffix, depends);
 		if (sb != gbuf)
 			sp = gbuf + (sp - sb);
+		sb = gbuf;
 		sub_ptr = sp;
 		if (*s2) {
 			if (strlcpy(name, s2, nlen) >= nlen) {
@@ -1398,6 +1503,7 @@ patssub(name, f1, f2, t1, t2)
 	if (sb != gbuf) {
 		sp = gbuf + (sp - sb);
 		sub_ptr = gbuf + (sub_ptr - sb);
+		sb = gbuf;
 	}
 	sub_s_put(&sp[1]);
 }
@@ -1405,7 +1511,7 @@ patssub(name, f1, f2, t1, t2)
 
 /*
  * Do pattern substitution in a macro that may contain multiple words.
- * Subtitution is applied separately to each word.
+ * Substitution is applied separately to each word.
  *
  * The parameter 'name' is expected to be on the 'growable buffer'.
  * If we ever need to use patmsub() otherwise, we need to add a boolean
@@ -1445,15 +1551,25 @@ patmsub(name, f1, f2, t1, t2)
 		if (sb != gbuf) {
 			sp = gbuf + (sp - sb);
 			osp = gbuf + (osp - sb);
-			name = gbuf + (name - sb);
-			if (b != NULL)
-				b = gbuf + (b - sb);
+			if (wason_gbuf(name)) {
+				name = gbuf + (name - sb);
+				if (b != NULL)
+					b = gbuf + (b - sb);
+			}
 			sb = gbuf;
 			sbe = gbufend;
 		}
 
 		if (b) {
 			sub_c_put(c);
+			if (sb != gbuf) {
+				sp = gbuf + (sp - sb);
+				osp = gbuf + (osp - sb);
+				if (wason_gbuf(b))
+					b = gbuf + (b - sb);
+				sb = gbuf;
+				sbe = gbufend;
+			}
 			while (*b != '\0' && white(*b)) {
 				sub_put(b++, 1);
 				if (sb != gbuf) {
@@ -1473,6 +1589,8 @@ patmsub(name, f1, f2, t1, t2)
 	if (sb != gbuf) {
 		sp = gbuf + (sp - sb);
 		sub_ptr = gbuf + (sub_ptr - sb);
+		sb = gbuf;
+		sbe = gbufend;
 	}
 	sub_s_put(&sp[1]);
 }
@@ -1544,7 +1662,12 @@ shout(cmd)
 	char	*sptr = sub_ptr;
 	char	*sbuf = gbuf;
 
-	if (sub_ptr == NULL)
+	/*
+	 * getshvar() does not know about sub_ptr and thus cannot initialize
+	 * it. As it first overwrites gbuf and then calls us with name == gbuf,
+	 * we can do the needed initialization for sub_c_put() here.
+	 */
+	if (sub_ptr == NULL || cmd == gbuf)
 		sptr = sub_ptr = gbuf;
 
 	f = popen(cmd, "r");
@@ -1561,6 +1684,7 @@ shout(cmd)
 	*sub_ptr = '\0';
 	if (sbuf != gbuf)
 		sptr = gbuf + (sptr - sbuf);
+	sbuf = gbuf;
 	return (sptr);
 }
 
@@ -1592,12 +1716,13 @@ shsub(l, obj, source, suffix, depends)
 	if (sbuf != gbuf)
 		sptr = gbuf + (sptr - sbuf);
 	sub_ptr = sptr;
+	sbuf = gbuf;
 
 	return (shout(sptr));
 }
 
 /*
- * Expand a macro for which the name is explicitely known.
+ * Expand a macro for which the name is explicitly known.
  */
 LOCAL void
 exp_name(name, obj, source, suffix, depends, pat)
@@ -1627,8 +1752,8 @@ exp_name(name, obj, source, suffix, depends, pat)
 		o = objlook(name, FALSE);
 		if (o == (obj_t *)NULL) {
 			/*
-			 * Check for $O -> ObjDir (speudo dyn) macro.
-			 * $O is a speudo dyn macro as we allow to overwrite it
+			 * Check for $O -> ObjDir (pseudo dyn) macro.
+			 * $O is a pseudo dyn macro as we allow to overwrite it
 			 * in order get full POSIX make compliance.
 			 */
 			if (name[0] == 'O' && name[1] == '\0') {
@@ -1658,6 +1783,7 @@ exp_name(name, obj, source, suffix, depends, pat)
 						parsepat(epa, &f1, &f2, &t1, &t2);
 						if (sb != gbuf)
 							sp = gbuf + (sp - sb);
+						sb = gbuf;
 						/*
 						 * patmsub() expects first
 						 * parameter to be in 'gbuf'
@@ -1692,6 +1818,7 @@ exp_name(name, obj, source, suffix, depends, pat)
 			if (sb != gbuf)
 				sp = gbuf + (sp - sb);
 			sub_ptr = sp;
+			sb = gbuf;
 			(void) shout(sp);
 		}
 		return;
@@ -1720,6 +1847,7 @@ exp_name(name, obj, source, suffix, depends, pat)
 		if (ispat) {
 			if (sb != gbuf)
 				sp = gbuf + (sp - sb);
+			sb = gbuf;
 			/*
 			 * patmsub() expects first parameter to be in 'gbuf'
 			 */
@@ -1738,6 +1866,7 @@ exp_name(name, obj, source, suffix, depends, pat)
 		if (ispat) {
 			if (sb != gbuf)
 				sp = gbuf + (sp - sb);
+			sb = gbuf;
 			/*
 			 * patmsub() expects first parameter to be in 'gbuf'
 			 */
@@ -1884,7 +2013,7 @@ patr_src(name, prule, rtypep, suffixp, pcmd, dlev)
 	cmd_t	**pcmd;
 	int	dlev;
 {
-		char	*xname = name;
+		char	*xname;
 	register obj_t	*source;
 		obj_t	newsource;
 		char	_sourcename[TYPICAL_NAMEMAX];
@@ -1914,7 +2043,7 @@ again:
 	*suffixp = (char *)prule->p_tgt_suffix;
 
 	p = (char *)prule->p_tgt_prefix;
-	xname += prule->p_tgt_pfxlen;	/* strip matching prefix */
+	xname = name + prule->p_tgt_pfxlen;	/* strip matching prefix */
 /*error("name: '%s'\n", xname);*/
 	p = rstr(xname, (char *)prule->p_tgt_suffix);
 /*error("p: '%s'\n", p);*/
@@ -2117,8 +2246,10 @@ one_suff_src(name, suffix, pcmd, dlev)
 						"suffix source name");
 			if (sp == NULL) {
 				/*
-				 * Copy old content
+				 * Copy old content.
+				 * First null-terminate the current buffer.
 				 */
+				_sourcename[sizeof (_sourcename) - 1] = '\0';
 				strlcpy(sourcename, _sourcename, sourcelen+1);
 			}
 			newsource.o_name = sp = sourcename;
@@ -2218,8 +2349,10 @@ ssuff_src(name, rule, rtypep, suffixp, pcmd, dlev)
 						"simple suffix source name");
 			if (sp == NULL) {
 				/*
-				 * Copy old content
+				 * Copy old content.
+				 * First null-terminate the current buffer.
 				 */
+				_sourcename[sizeof (_sourcename) - 1] = '\0';
 				strlcpy(sourcename, _sourcename, sourcelen+1);
 			}
 			newsource.o_name = sp = sourcename;
@@ -2341,7 +2474,7 @@ default_cmd(obj, depname, deptime, deplevel, must_exist, dlev)
 		errmsgno(EX_BAD,
 			"Recursion in default rule for '%s'.\n",
 			obj->o_name);
-		return (NOTIME);
+		return (RECURSETIME);
 	}
 
 	if (((obj->o_flags & F_TERM) == 0) &&
@@ -2582,10 +2715,10 @@ make(obj, must_exist, dlev)
 		if (obj->o_flags & F_DCOLON)
 			dcolon_time(obj);
 		/*
-		 * Fake sucsess for targets listed in the makefile that have
-		 * no exlicit commands no explicit dependencies (prerequisites)
+		 * Fake success for targets listed in the makefile that have
+		 * no explicit commands no explicit dependencies (prerequisites)
 		 * and where we could not find implicit dependencies.
-		 * These intermediate placeholde targets look similar to FORCE:
+		 * These intermediate placeholder targets look similar to FORCE:
 		 */
 		if (obj->o_list == NULL && obj->o_date == NOTIME && must_exist)
 			obj->o_date = newtime;
